package io.legado.app.model.localBook

import android.net.Uri
import androidx.documentfile.provider.DocumentFile
import io.legado.app.data.appDb
import io.legado.app.data.entities.Book
import io.legado.app.data.entities.BookChapter
import io.legado.app.help.BookHelp
import io.legado.app.utils.*
import splitties.init.appCtx
import java.io.File
import java.util.regex.Pattern
import io.legado.app.constant.AppConst
import io.legado.app.constant.AppPattern
import io.legado.app.help.AppConfig
import javax.script.SimpleBindings
import java.util.regex.Matcher

object LocalBook {
    private const val folderName = "bookTxt"
    val cacheFolder: File by lazy {
        val rootFile = appCtx.getExternalFilesDir(null)
            ?: appCtx.externalCacheDir
            ?: appCtx.cacheDir
        FileUtils.createFolderIfNotExist(rootFile, folderName)
    }

    fun getChapterList(book: Book): ArrayList<BookChapter> {
        return if (book.isEpub()) {
            EpubFile.getChapterList(book)
        } else {
            AnalyzeTxtFile().analyze(book)
        }
    }

    fun getContext(book: Book, chapter: BookChapter): String? {
        return if (book.isEpub()) {
            EpubFile.getContent(book, chapter)
        } else {
            AnalyzeTxtFile.getContent(book, chapter)
        }
    }

    fun importFile(uri: Uri): Book {
        val path: String
        val fileName = (if (uri.isContentScheme()) {
            path = uri.toString()
            val doc = DocumentFile.fromSingleUri(appCtx, uri)
            doc?.let {
                val bookFile = FileUtils.getFile(cacheFolder, it.name!!)
                if (!bookFile.exists()) {
                    bookFile.createNewFile()
                    doc.readBytes(appCtx)?.let { bytes ->
                        bookFile.writeBytes(bytes)
                    }
                }
            }
            doc?.name!!
        } else {
            path = uri.path!!
            File(path).name
        }).replace(Regex("\\.txt$"), "")

        val name: String
        val author: String

        //匹配(知轩藏书常用格式) 《书名》其它信息作者：作者名.txt
        val m1 = Pattern
            .compile("(.*?)《([^《》]+)》(.*)")
            .matcher(fileName)

        //匹配 书名 by 作者名.txt
        val m2 = Pattern
            .compile("(^)(.+) by (.+)$")
            .matcher(fileName)

        (m1.takeIf { m1.find() }
            ?: m2.takeIf { m2.find() }
                ). run{

                if(this is Matcher) {

                    //按默认格式将文件名分解成书名、作者名
                    name = group(2)!!
                    author = BookHelp.formatBookAuthor((group(1) ?: "") + (group(3) ?: ""))

                } else if(!AppConfig.bookImportFileName.isNullOrBlank()) {

                    //在脚本中定义如何分解文件名成书名、作者名
                    val bindings = SimpleBindings()
                    bindings["src"] = fileName
                    val jsonStr = AppConst.SCRIPT_ENGINE.eval(

                        //在用户脚本后添加捕获author、name的代码，只要脚本中author、name有值就会被捕获，未定义则赋值为空字符串
                        AppConfig.bookImportFileName + "\nJSON.stringify({author:author,name:name})"
                        , bindings).toString()
                    val bookMess =GSON.fromJsonObject<HashMap<String, String>>(jsonStr) ?: HashMap()
                    name = bookMess["name"]?: fileName
                    author = bookMess["author"]?.takeIf { it.length != fileName.length } ?: ""

                } else {

                    name = fileName.replace(AppPattern.nameRegex,"")
                    author = fileName.replace(AppPattern.authorRegex,"").takeIf { it.length != fileName.length } ?: ""

                }

            }

        val book = Book(
            bookUrl = path,
            name = name,
            author = author,
            originName = fileName,
            coverUrl = FileUtils.getPath(
                appCtx.externalFilesDir,
                "covers",
                "${MD5Utils.md5Encode16(path)}.jpg"
            )
        )
        if (book.isEpub()) EpubFile.upBookInfo(book)
        appDb.bookDao.insert(book)
        return book
    }

    fun deleteBook(book: Book, deleteOriginal: Boolean) {
        kotlin.runCatching {
            if (book.isLocalTxt()) {
                val bookFile = FileUtils.getFile(cacheFolder, book.originName)
                bookFile.delete()
            }
            if(book.isEpub()){
                val bookFile=BookHelp.getEpubFile(book).parentFile
                if (bookFile!=null&&bookFile.exists()){
                    FileUtils.delete(bookFile,true)
                }

            }

            if (deleteOriginal) {
                if (book.bookUrl.isContentScheme()) {
                    val uri = Uri.parse(book.bookUrl)
                    DocumentFile.fromSingleUri(appCtx, uri)?.delete()
                } else {
                    FileUtils.deleteFile(book.bookUrl)
                }
            }
        }
    }
}
